from fastapi import APIRouter, WebSocket, WebSocketDisconnect
from fer import FER
import cv2
import imutils 
from scripts.blink_detection import count_blinks
from baseModels.payloads import CountBlinks
from scripts.web_socket import ConnectionManager
import json

manager = ConnectionManager()
router = APIRouter(prefix='/facial')

@router.post("/eye-blinks")
def countBlinks(payload:CountBlinks):
    video_filename = 'videos/interviews/'+payload.application_id+'.mp4'
    count = count_blinks(video_filename, payload.start, payload.end)
    return count

@router.websocket("/ws/eye-blinks")
async def countBlinks(websocket: WebSocket,  application_id: str, start:str, end:str):
    await manager.connect(websocket)
    try:
        while True:
            video_filename = 'videos/interviews/'+application_id+'.mp4'
            count = count_blinks(video_filename, int(start), int(end))
            await manager.send_private(json.dumps({"count":count, "end":True}), websocket)
            await manager.disconnect(websocket)
            
    except WebSocketDisconnect:
        await manager.send_private(json.dumps({"end":True}), websocket)
        await manager.disconnect(websocket)
    return

@router.websocket("/ws/emotions")
async def emotions(websocket: WebSocket, application_id: str):
    await manager.connect(websocket)
    video_filename = 'videos/interviews/'+application_id+'.mp4'
    face_detector = FER(mtcnn=True)
    cap = cv2.VideoCapture(video_filename)
    try:
        while True:
            ret, frame = cap.read()
            if not ret:
                await manager.send_private(json.dumps({"data":[], "end":True}), websocket)
                await manager.disconnect(websocket)
                break
            
            frame = imutils.resize(frame, width=800)
            emotions = face_detector.detect_emotions(frame)
            print(emotions)
            await manager.send_private(json.dumps({"data":emotions, "end":False}), websocket)
            
    except WebSocketDisconnect:
        await manager.send_private(json.dumps({"data":[], "end":True, "status":"Offline"}), websocket)
        await manager.disconnect(websocket)


